"""Tests for the templatetags of the ``subscriptions`` app."""
from django.contrib.auth.models import AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from mixer.backend.django import mixer

from ..templatetags.subscribe_tags import (
    get_ctype,
    get_subscribers,
    is_subscribed,
)


class GetCtypeTestCase(TestCase):
    """Tests for the ``get_ctype`` templatetag."""
    def test_tag(self):
        dummy = mixer.blend('test_app.DummyModel')
        ctype = ContentType.objects.get_for_model(dummy)
        result = get_ctype(dummy)
        self.assertEqual(result, ctype)


class GetSubscribersTestCase(TestCase):
    """Tests for the ``get_subscribers`` templatetag."""
    def test_tag(self):
        # Two subscriptions for the same thing
        sub1 = mixer.blend('subscribe.Subscription',
                           content_object=mixer.blend('test_app.DummyModel'))
        mixer.blend('subscribe.Subscription',
                    content_object=sub1.content_object)

        # One subscription for another thing
        mixer.blend('subscribe.Subscription')

        result = get_subscribers(sub1.content_object)
        self.assertEqual(result.count(), 2)


class IsSubscribedTestCase(TestCase):
    """Tests for the ``is_subscribed`` templatetag."""
    def test_is_anonymous(self):
        sub = mixer.blend('subscribe.Subscription',
                          content_object=mixer.blend('test_app.DummyModel'))
        result = is_subscribed(AnonymousUser(), sub.content_object)
        self.assertFalse(result)

    def test_is_subscribed(self):
        sub = mixer.blend('subscribe.Subscription',
                          content_object=mixer.blend('test_app.DummyModel'))
        result = is_subscribed(sub.user, sub.content_object)
        self.assertTrue(result)

    def test_is_not_subscribed(self):
        sub1 = mixer.blend('subscribe.Subscription',
                           content_object=mixer.blend('test_app.DummyModel'))
        sub2 = mixer.blend('subscribe.Subscription',
                           content_object=mixer.blend('test_app.DummyModel'))
        result = is_subscribed(sub1.user, sub2.content_object)
        self.assertFalse(result)



"""Views of the ``subscribe`` app."""
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.http import Http404
from django.utils.decorators import method_decorator
from django.views.generic import FormView

from .forms import SubscriptionCreateForm, SubscriptionDeleteForm


class SubscriptionCreateView(FormView):
    """View that subscribes a ``User`` to any thing."""
    form_class = SubscriptionCreateForm
    template_name = 'subscribe/subscription_form.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.user = request.user

        try:
            self.ctype = ContentType.objects.get(pk=kwargs.get('ctype_pk'))
        except ContentType.DoesNotExist:
            return Http404

        try:
            self.content_object = self.ctype.get_object_for_this_type(
                pk=kwargs.get('object_pk'))
        except ObjectDoesNotExist:
            return Http404

        return super(SubscriptionCreateView, self).dispatch(
            request, *args, **kwargs)

    def form_valid(self, form):
        self.object = form.save()
        return super(SubscriptionCreateView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        ctx = super(SubscriptionCreateView, self).get_context_data(**kwargs)
        ctx.update({
            'content_object': self.content_object,
        })
        return ctx

    def get_form_kwargs(self):
        kwargs = super(SubscriptionCreateView, self).get_form_kwargs()
        kwargs.update({
            'user': self.user,
            'content_object': self.content_object,
        })
        return kwargs

    def get_success_url(self):
        return self.content_object.get_absolute_url()


class SubscriptionDeleteView(SubscriptionCreateView):
    """View that un-subscribes a ``User`` from any thing."""
    form_class = SubscriptionDeleteForm
    template_name = 'subscribe/subscription_delete.html'



"""Tests for the forms of the ``subscriptions`` app."""
from django.test import TestCase

from mixer.backend.django import mixer

from ..forms import SubscriptionCreateForm, SubscriptionDeleteForm
from ..models import Subscription


class SubscriptionCreateFormTestCase(TestCase):
    """Tests for the ``SubscriptionCreateForm`` form class."""
    longMessage = True

    def test_save(self):
        """Should create a new subscription."""
        user = mixer.blend('auth.User')
        dummy = mixer.blend('test_app.DummyModel')
        form = SubscriptionCreateForm(user=user, content_object=dummy, data={})
        self.assertTrue(form.is_valid(), msg=(
            'Errors: {0}'.format(form.errors.items())))
        instance = form.save()
        self.assertTrue(instance.pk)


class SubscriptionDeleteFormTestCase(TestCase):
    """Tests for the ``SubscriptionDeleteForm`` form class."""
    longMessage = True

    def test_save(self):
        """Should delete the subscription."""
        sub = mixer.blend('subscribe.Subscription',
                          content_object=mixer.blend('test_app.DummyModel'))
        form = SubscriptionDeleteForm(
            user=sub.user, content_object=sub.content_object, data={})
        self.assertTrue(form.is_valid(), msg=(
            'Errors: {0}'.format(form.errors.items())))
        form.save()
        self.assertEqual(Subscription.objects.all().count(), 0)

    def test_no_subscription(self):
        """
        Should fail graciously if trying to delete a non existant subscription.

        """
        sub = mixer.blend('subscribe.Subscription',
                          content_object=mixer.blend('test_app.DummyModel'))
        form = SubscriptionDeleteForm(
            user=sub.user, content_object=sub.content_object, data={})
        self.assertTrue(form.is_valid(), msg=(
            'Errors: {0}'.format(form.errors.items())))
        sub.delete()
        form.save()
        self.assertEqual(Subscription.objects.all().count(), 0)



"""Dummy models for tests of the ``subscription`` app."""
from django.db import models


class DummyModel(models.Model):
    name = models.CharField(max_length=256)

    def get_absolute_url(self):
        return '/'



"""URLs for the ``subscribe`` app."""
from django.conf.urls import url

from .views import SubscriptionCreateView, SubscriptionDeleteView


urlpatterns = [
    url(r'^create/(?P<ctype_pk>\d+)/(?P<object_pk>\d+)/$',
        SubscriptionCreateView.as_view(),
        name='subscriptions_create',),
    url(r'^delete/(?P<ctype_pk>\d+)/(?P<object_pk>\d+)/$',
        SubscriptionDeleteView.as_view(),
        name='subscriptions_delete',),
]



"""Tests for the views of the ``subscriptions`` app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin
from mixer.backend.django import mixer

from .. import views


class SubscriptionCreateViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    view_class = views.SubscriptionCreateView

    def setUp(self):
        self.user = mixer.blend('auth.User')
        self.dummy = mixer.blend('test_app.DummyModel')
        self.ctype = ContentType.objects.get_for_model(self.dummy)

    def get_view_kwargs(self):
        return {'ctype_pk': self.ctype.pk, 'object_pk': self.dummy.pk, }

    def test_anonymous(self):
        """Should redirect to login if user is anonymous."""
        self.should_redirect_to_login_when_anonymous()

    def test_callable(self):
        """Should be callable if user is authenticated."""
        self.is_callable(user=self.user)


class SubscriptionDeleteViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    view_class = views.SubscriptionDeleteView

    def setUp(self):
        self.subscription = mixer.blend(
            'subscribe.Subscription',
            content_object=mixer.blend('test_app.DummyModel'))
        self.ctype = ContentType.objects.get_for_model(
            self.subscription.content_type)

    def get_view_kwargs(self):
        return {
            'ctype_pk': self.subscription.content_type.pk,
            'object_pk': self.subscription.object_id,
        }

    def test_anonymous(self):
        """Should redirect to login if user is anonymous."""
        self.should_redirect_to_login_when_anonymous()

    def test_callable(self):
        """Should be callable if user is authenticated."""
        self.is_callable(user=self.subscription.user)
        self.is_postable(user=self.subscription.user, to='/')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-20 08:44
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscribed', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='subscription',
            unique_together=set([('user', 'content_type', 'object_id')]),
        ),
    ]



"""Models for the ``subscribe`` app."""
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Subscription(models.Model):
    """
    Allows a ``User`` to subscribe to anything.

    :user: The ``User`` who subscribed to something.
    :content_object: Generic foreign key to the thing that the user is
      subscribed to.
    :date: Date when the subscription was created.

    """
    class Meta:
        unique_together = ('user', 'content_type', 'object_id', )

    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='subscriptions',
    )

    content_type = models.ForeignKey(
        ContentType,
        related_name='subscribed',
    )
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    creation_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_('Creation date'),
    )

    def __str__(self):
        return u'{0} subscribed to {1}'.format(
            self.user.email, self.content_object)



import os
from setuptools import setup, find_packages
import subscribe


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''


setup(
    name="django-subscribe",
    version=subscribe.__version__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    license='The MIT License',
    platforms=['OS Independent'],
    keywords='django, subsription, generic',
    author='Martin Brochhaus',
    author_email='mbrochh@gmail.com',
    url="https://github.com/bitmazk/django-subscribe",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django',
    ],
)



"""Templatetags for the ``subscribe`` app."""
from django import template
from django.contrib.contenttypes.models import ContentType

from ..models import Subscription


register = template.Library()


@register.assignment_tag
def get_ctype(obj):
    """
    Returns the ``ContentType`` for the given object.

    :param obj: Any object.

    """
    return ContentType.objects.get_for_model(obj)


@register.assignment_tag
def get_subscribers(obj):
    """
    Returns the subscribers for a given object.

    :param obj: Any object.

    """
    ctype = ContentType.objects.get_for_model(obj)
    return Subscription.objects.filter(content_type=ctype, object_id=obj.pk)


@register.assignment_tag
def is_subscribed(user, obj):
    """
    Returns ``True`` if the user is subscribed to the given object.

    :param user: A ``User`` instance.
    :param obj: Any object.

    """
    if not user.is_authenticated():
        return False

    ctype = ContentType.objects.get_for_model(obj)

    try:
        Subscription.objects.get(
            user=user, content_type=ctype, object_id=obj.pk)
    except Subscription.DoesNotExist:
        return False

    return True



"""Tests for the models of the ``subscriptions`` app."""
from django.test import TestCase

from mixer.backend.django import mixer


class SubscriptionTestCase(TestCase):
    """Tests for the ``Subscription`` model class."""
    def test_model(self):
        """Should be able to instantiate and save the model."""
        instance = mixer.blend('subscribe.Subscription')
        self.assertTrue(str(instance))
